from firebase_admin import auth

from package_tests.models import User


# Stubs

stub_firebase_token = 'stub_firebase_token'
stub_firebase_uid = 'stub_firebase_uid'

stub_email = 'daniel@danieljs.tech'
stub_username = 'dspacejs'


# Mock classes

class MockUserRecord(object):
    email = None
    display_name = None
    uid = None

    def __init__(self, email, display_name, uid):
        self.email = email
        self.display_name = display_name
        self.uid = uid


# Mock methods

def mock_firebase_verify_id_token(encoded_token, app, check_revoked):
    if encoded_token == stub_firebase_token:
        return {
            'uid': stub_firebase_uid,
        }
    elif encoded_token is None:
        raise ValueError('Token is not valid.')
    elif encoded_token == '':
        raise ValueError('Token is not valid.')
    raise auth.AuthError(code='USER_NOT_FOUND', message='User not found.')

def mock_firebase_get_user(firebase_uid):
    if firebase_uid == stub_firebase_uid:
        return MockUserRecord(
            email=stub_email,
            display_name=stub_username,
            uid=stub_firebase_uid,
        )
    raise auth.AuthError(code='USER_NOT_FOUND', message='User not found.')

def create_user(firebase_uid=stub_firebase_uid, email='email@gmail.com'):
    user = User.objects.create(email=email, firebase_uid=firebase_uid)
    user.set_password('supersekret')
    user.save()
    return user

def setup_mocks():
    auth.verify_id_token = mock_firebase_verify_id_token
    auth.get_user = mock_firebase_get_user



from django.apps import AppConfig
from django.conf import settings
import firebase_admin


firebase_app = None


class FirebaseAuthConfig(AppConfig):
    name = 'firebase_auth'

    def ready(self):
        credentials = firebase_admin.credentials.Certificate(
            settings.GOOGLE_APPLICATION_CREDENTIALS,
        )
        global firebase_app
        firebase_app = firebase_admin.initialize_app(credentials)



from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models

from firebase_auth.models import FirebaseAuthMixin


class User(AbstractBaseUser, PermissionsMixin, FirebaseAuthMixin):
    email = models.EmailField(unique=True)
    username = models.CharField(max_length=50, unique=True)

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email',]
